@@ -1,6 +1,8 @@
 from typing import Union, Dict, List, Tuple
-from amulet import log
 from enum import Enum
+import logging
+
+log = logging.getLogger(__name__)
 
 
 class StatusFormats(Enum):
