@@ -1,7 +1,7 @@
 from __future__ import annotations
 import os
 
-from amulet_nbt import CompoundTag, load as load_nbt
+from amulet_nbt import load as load_nbt
 
 from .anvil_world import AnvilFormat
 from amulet.utils.format_utils import check_all_exist
