@@ -14,6 +14,7 @@ import numpy
 from io import BytesIO
 import struct
 import copy
+import logging
 
 from amulet_nbt import (
     ByteTag,
@@ -27,7 +28,7 @@ from amulet_nbt import (
     load as load_nbt,
 )
 
-from amulet import log
+
 from amulet.api.data_types import (
     AnyNDArray,
     VersionNumberAny,
@@ -57,6 +58,8 @@ from .util import (
 if TYPE_CHECKING:
     from amulet.api.wrapper import Translator, Interface
 
+log = logging.getLogger(__name__)
+
 construction_0_interface = Construction0Interface()
 
 INT_STRUCT = struct.Struct(">I")
@@ -349,28 +352,24 @@ class ConstructionFormatWrapper(StructureFormatWrapper[VersionNumberTuple]):
         f.write(magic_num)
         f.write(struct.pack(">B", self._format_version))
         if self._format_version == 0:
-            metadata = NamedTag(
-                CompoundTag(
-                    {
-                        "created_with": StringTag("amulet_python_wrapper_v2"),
-                        "selection_boxes": IntArrayTag(
-                            [
-                                c
-                                for box in self._bounds[
-                                    self.dimensions[0]
-                                ].selection_boxes
-                                for c in (*box.min, *box.max)
-                            ]
-                        ),
-                        "section_version": ByteTag(self._section_version),
-                        "export_version": CompoundTag(
-                            {
-                                "edition": StringTag(self._platform),
-                                "version": ListTag([IntTag(v) for v in self._version]),
-                            }
-                        ),
-                    }
-                )
+            metadata = CompoundTag(
+                {
+                    "created_with": StringTag("amulet_python_wrapper_v2"),
+                    "selection_boxes": IntArrayTag(
+                        [
+                            c
+                            for box in self._bounds[self.dimensions[0]].selection_boxes
+                            for c in (*box.min, *box.max)
+                        ]
+                    ),
+                    "section_version": ByteTag(self._section_version),
+                    "export_version": CompoundTag(
+                        {
+                            "edition": StringTag(self._platform),
+                            "version": ListTag([IntTag(v) for v in self._version]),
+                        }
+                    ),
+                }
             )
             section_index_table: List[
                 Tuple[int, int, int, int, int, int, int, int]
@@ -403,7 +402,7 @@ class ConstructionFormatWrapper(StructureFormatWrapper[VersionNumberTuple]):
                             )
                             flattened_array = lut[flattened_array]
                             array_type = find_fitting_array_type(flattened_array)
-                            _tag["blocks_array_type"] = ByteTag(array_type().tag_id)
+                            _tag["blocks_array_type"] = ByteTag(array_type.tag_id)
                             _tag["blocks"] = array_type(flattened_array)
                             _tag["block_entities"] = serialise_block_entities(
                                 block_entities or []
