@@ -27,7 +27,7 @@ from amulet.api.player import Player, LOCAL_PLAYER
 from amulet.api.chunk import Chunk
 from amulet.api.selection import SelectionBox, SelectionGroup
 
-from amulet.libs.leveldb import LevelDB, LevelDBException, LevelDBEncrypted
+from leveldb import LevelDB, LevelDBException, LevelDBEncrypted
 from amulet.utils.format_utils import check_all_exist
 from amulet.api.data_types import (
     ChunkCoordinates,
@@ -94,7 +94,7 @@ class BedrockLevelDAT(NamedTag):
     def _read_from(path: str) -> Tuple[int, str, AbstractBaseTag]:
         with open(path, "rb") as f:
             level_dat_version = struct.unpack("<i", f.read(4))[0]
-            if 4 <= level_dat_version <= 9:
+            if 4 <= level_dat_version <= 10:
                 data_length = struct.unpack("<i", f.read(4))[0]
                 root_tag = load_nbt(
                     f.read(data_length),
@@ -167,7 +167,12 @@ class LevelDBFormat(WorldFormatWrapper[VersionNumberTuple]):
         """
         super().__init__(path)
         self._platform = "bedrock"
-        self._root_tag = BedrockLevelDAT.from_file(os.path.join(path, "level.dat"))
+        dat_path = os.path.join(path, "level.dat")
+        if os.path.isfile(dat_path):
+            self._root_tag = BedrockLevelDAT.from_file(dat_path)
+        else:
+            # TODO: handle level creation better
+            self._root_tag = BedrockLevelDAT(path=dat_path, level_dat_version=9)
         self._db = None
         self._dimension_manager = None
         self._shallow_load()
