@@ -30,6 +30,7 @@ from amulet.api.data_types import (
     SubChunkNDArray,
     PlatformType,
     VersionNumberTuple,
+    VersionIdentifierTuple,
 )
 from amulet.level import loader
 from amulet.api.wrapper import EntityIDType, EntityCoordType
@@ -140,14 +141,14 @@ class BaseLevelDBInterface(Interface):
                     ] = chunk_data.pop(key)
             chunk.blocks, chunk_palette = self._load_subchunks(subchunks)
         elif self._features["terrain"] == "30array":
-            chunk_data = chunk_data.pop(b"\x30", None)
-            if chunk_data is not None:
+            section_data = chunk_data.pop(b"\x30", None)
+            if section_data is not None:
                 block_ids = numpy.frombuffer(
-                    chunk_data[: 2**15], dtype=numpy.uint8
+                    section_data[: 2**15], dtype=numpy.uint8
                 ).astype(numpy.uint16)
                 block_data = from_nibble_array(
                     numpy.frombuffer(
-                        chunk_data[2**15 : 2**15 + 2**14], dtype=numpy.uint8
+                        section_data[2**15 : 2**15 + 2**14], dtype=numpy.uint8
                     )
                 )
 
@@ -243,7 +244,7 @@ class BaseLevelDBInterface(Interface):
         self,
         chunk: Chunk,
         palette: AnyNDArray,
-        max_world_version: VersionNumberTuple,
+        max_world_version: VersionIdentifierTuple,
         bounds: Tuple[int, int],
     ) -> Dict[bytes, Optional[bytes]]:
         chunk_data = chunk.misc.get("bedrock_chunk_data", {})
@@ -428,14 +429,17 @@ class BaseLevelDBInterface(Interface):
                     palette_data_out: List[Tuple[Optional[int], Block]] = []
                     for block in palette_data:
                         block = block.compound
-                        namespace, base_name = block["name"].py_str.split(":", 1)
+                        *namespace_, base_name = block["name"].py_str.split(":", 1)
+                        namespace = namespace_[0] if namespace_ else "minecraft"
                         if "version" in block:
                             version: Optional[int] = block.get_int("version").py_int
                         else:
                             version = None
 
-                        if "states" in block:  # 1.13 format
-                            properties = block.get_compound("states").py_dict
+                        if "states" in block or "val" not in block:  # 1.13 format
+                            properties = block.get_compound(
+                                "states", CompoundTag()
+                            ).py_dict
                             if version is None:
                                 version = 17694720  # 1, 14, 0, 0
                         else:
@@ -500,7 +504,7 @@ class BaseLevelDBInterface(Interface):
         blocks: "Blocks",
         palette: AnyNDArray,
         bounds: Tuple[int, int],
-        max_world_version: VersionNumberTuple,
+        max_world_version: VersionIdentifierTuple,
     ) -> Dict[int, Optional[bytes]]:
         raise NotImplementedError
 
