@@ -1,5 +1,15 @@
 import os
-from typing import Optional, Tuple, Iterable, TYPE_CHECKING, BinaryIO, Dict, Union
+from typing import (
+    Optional,
+    Tuple,
+    Iterable,
+    TYPE_CHECKING,
+    BinaryIO,
+    Dict,
+    Union,
+    List,
+    NamedTuple,
+)
 import numpy
 import copy
 
@@ -32,7 +42,6 @@ from .interface import (
     BedrockSchematicInterface,
     SchematicInterface,
 )
-from .data_types import BlockArrayType, BlockDataArrayType
 from .chunk import SchematicChunk
 
 if TYPE_CHECKING:
@@ -58,7 +67,7 @@ class SchematicFormatWrapper(StructureFormatWrapper[VersionNumberTuple]):
         super().__init__(path)
         self._chunks: Dict[
             ChunkCoordinates,
-            Tuple[SelectionBox, BlockArrayType, BlockDataArrayType, list, list],
+            SchematicChunk,
         ] = {}
 
     def _create(
@@ -139,23 +148,26 @@ class SchematicFormatWrapper(StructureFormatWrapper[VersionNumberTuple]):
                     min((cz + 1) * self.sub_chunk_size, selection_box.size_z),
                 ),
             )
-            self._chunks[(cx, cz)] = (box, blocks[box.slice], data[box.slice], [], [])
+            self._chunks[(cx, cz)] = SchematicChunk(
+                box, blocks[box.slice], data[box.slice], [], []
+            )
         for e in block_entities:
-            if all(key in e for key in ("x", "y", "z")):
+            if isinstance(e, CompoundTag) and all(key in e for key in ("x", "y", "z")):
                 x = e.get_int("x").py_int
                 y = e.get_int("y").py_int
                 z = e.get_int("z").py_int
                 if (x, y, z) in selection_box:
                     cx = x >> 4
                     cz = z >> 4
-                    self._chunks[(cx, cz)][3].append(e)
+                    self._chunks[(cx, cz)].block_entities.append(e)
         for e in entities:
-            pos: PointCoordinates = tuple(map(float, e.get_list("Pos", ListTag())))
-            if len(pos) == 3:
-                if pos in selection_box:
-                    cx = int(pos[0]) >> 4
-                    cz = int(pos[2]) >> 4
-                    self._chunks[(cx, cz)][4].append(e)
+            if isinstance(e, CompoundTag) and "Pos" in e:
+                pos: PointCoordinates = tuple(map(float, e.get_list("Pos", ListTag())))
+                if len(pos) == 3:
+                    if pos in selection_box:
+                        cx = int(pos[0]) >> 4
+                        cz = int(pos[2]) >> 4
+                        self._chunks[(cx, cz)].entities.append(e)
 
     @staticmethod
     def is_valid(path: str) -> bool:
@@ -198,17 +210,14 @@ class SchematicFormatWrapper(StructureFormatWrapper[VersionNumberTuple]):
 
         selection = self._bounds[self.dimensions[0]].selection_boxes[0]
 
-        data = NamedTag(
-            CompoundTag(
-                {
-                    "TileTicks": ListTag(),
-                    "Width": ShortTag(selection.size_x),
-                    "Height": ShortTag(selection.size_y),
-                    "Length": ShortTag(selection.size_z),
-                    "Materials": StringTag(materials),
-                }
-            ),
-            "Schematic",
+        tag = CompoundTag(
+            {
+                "TileTicks": ListTag(),
+                "Width": ShortTag(selection.size_x),
+                "Height": ShortTag(selection.size_y),
+                "Length": ShortTag(selection.size_z),
+                "Materials": StringTag(materials),
+            }
         )
 
         entities = []
@@ -220,42 +229,34 @@ class SchematicFormatWrapper(StructureFormatWrapper[VersionNumberTuple]):
             selection.shape, dtype=numpy.uint8
         )  # only 4 bits are used
 
-        for (
-            selection_,
-            blocks_,
-            data_,
-            block_entities_,
-            entities_,
-        ) in self._chunks.values():
-            if selection_.intersects(selection):
-                box = selection_.create_moved_box(selection.min, subtract=True)
-                blocks[box.slice] = blocks_
-                block_data[box.slice] = data_
-                for be in block_entities_:
+        for chunk in self._chunks.values():
+            if chunk.selection.intersects(selection):
+                box = chunk.selection.create_moved_box(selection.min, subtract=True)
+                blocks[box.slice] = chunk.blocks
+                block_data[box.slice] = chunk.data
+                for be in chunk.block_entities:
                     coord_type = be["x"].__class__
-                    be["x"] = coord_type(be["x"] - selection.min_x)
-                    be["y"] = coord_type(be["y"] - selection.min_y)
-                    be["z"] = coord_type(be["z"] - selection.min_z)
+                    be["x"] = coord_type(be["x"].py_int - selection.min_x)
+                    be["y"] = coord_type(be["y"].py_int - selection.min_y)
+                    be["z"] = coord_type(be["z"].py_int - selection.min_z)
                     block_entities.append(be)
-                for e in entities_:
+                for e in chunk.entities:
                     coord_type = e["Pos"][0].__class__
                     e["Pos"][0] = coord_type(e["Pos"][0] - selection.min_x)
                     e["Pos"][1] = coord_type(e["Pos"][1] - selection.min_y)
                     e["Pos"][2] = coord_type(e["Pos"][2] - selection.min_z)
                     entities.append(e)
 
-        data["Entities"] = ListTag(entities)
-        data["TileEntities"] = ListTag(block_entities)
-        data["Data"] = ByteArrayTag(
-            numpy.transpose(block_data, (1, 2, 0))  # XYZ => YZX
-        )
-        data["Blocks"] = ByteArrayTag(
+        tag["Entities"] = ListTag(entities)
+        tag["TileEntities"] = ListTag(block_entities)
+        tag["Data"] = ByteArrayTag(numpy.transpose(block_data, (1, 2, 0)))  # XYZ => YZX
+        tag["Blocks"] = ByteArrayTag(
             numpy.transpose((blocks & 0xFF).astype(numpy.uint8), (1, 2, 0))
         )
         if numpy.max(blocks) > 0xFF:
             add_blocks = (numpy.transpose(blocks & 0xF00, (1, 2, 0)) >> 8).ravel()
-            data["AddBlocks"] = ByteArrayTag((add_blocks[::2] << 4) + add_blocks[1::2])
-        data.save_to(f)
+            tag["AddBlocks"] = ByteArrayTag((add_blocks[::2] << 4) + add_blocks[1::2])
+        NamedTag(tag, "Schematic").save_to(f)
 
     def _close(self):
         """Close the disk database"""
@@ -343,15 +344,7 @@ class SchematicFormatWrapper(StructureFormatWrapper[VersionNumberTuple]):
         section: SchematicChunk,
         dimension: Optional[Dimension] = None,
     ):
-        self._chunks[(cx, cz)] = copy.deepcopy(
-            (
-                section.selection,
-                section.blocks,
-                section.data,
-                section.block_entities,
-                section.entities,
-            )
-        )
+        self._chunks[(cx, cz)] = copy.deepcopy(section)
 
     def _get_raw_chunk_data(
         self, cx: int, cz: int, dimension: Optional[Dimension] = None
@@ -365,6 +358,6 @@ class SchematicFormatWrapper(StructureFormatWrapper[VersionNumberTuple]):
         :return: The raw chunk data.
         """
         if (cx, cz) in self._chunks:
-            return SchematicChunk(*copy.deepcopy(self._chunks[(cx, cz)]))
+            return copy.deepcopy(self._chunks[(cx, cz)])
         else:
             raise ChunkDoesNotExist
